@@ -1,7 +1,7 @@
 
 /* dynamic_load - ͭ饤֥Υɤȼ¹ */
 
-/* $Id: dynamic_load.c,v 1.1.1.1 2003/05/13 08:17:15 tosihisa Exp $ */
+/* $Id: dynamic_load.c,v 1.2 2003/05/19 05:36:37 tosihisa Exp $ */
 
 #include <stdio.h>
 #ifdef HAVE_DL
@@ -15,6 +15,7 @@ int tnetio_dluse(void)
 #ifdef HAVE_DL
 	return 1;	/* ͭ饤֥굡ǽϻѲǽ */
 #else
+	tnetio_log_write(" ABORT : Not supported : Dynamic-link library function.\n");
 	return 0;	/* ͭ饤֥굡ǽϻѤǤʤ */
 #endif
 }
@@ -22,7 +23,7 @@ int tnetio_dluse(void)
 void *tnetio_dlopen(char *libname)
 {
 #ifdef HAVE_DL
-	return dlopen(libname,RTLD_LAZY);
+	return dlopen(libname,RTLD_NOW /* RTLD_LAZY */);
 #else
 	return NULL;
 #endif
