@@ -3,7 +3,7 @@
 
 /* tnetsrv [options] port-num program [program-arg(s)] */
 
-/* $Id: tnetsrv.c,v 1.4 2003/05/14 08:14:25 tosihisa Exp $ */
+/* $Id: tnetsrv.c,v 1.6 2003/05/20 09:59:57 tosihisa Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,6 +26,9 @@
 
 #include <errno.h>
 
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include "tnetio.h"
 
 static const char *log_msg[] = {
@@ -36,10 +39,10 @@ static const char *log_msg[] = {
 	" wait    : : (%d/%d)\n",
 	" bind    : %s %d %s : %s\n",
 	" signal  : %d : (%d/%d)\n",
-	" WARNING : IP_OPTION : [%s]\n",
+	NULL,
 	" ERROR   : exec(%s) : [%s]\n",
 	" ERROR   : fork() : [%s]\n",
-	" ERROR   : dynamic-link (%s) : [%s]\n",
+	" ABORT   : dynamic-link (%s) : [%s]\n",
 	NULL
 };
 
@@ -62,32 +65,46 @@ struct _client_info {	/* 饤(ҥץ) */
 
 struct _server_info {
 	struct sockaddr_in addr;
-	/* struct servent ent; */
+	int type;
 };
 
 struct _total_info {
-	int connect;			/* ³߷(reject ޤ)     */
+	int now_connect;		/* ߤƱ³ϿƤ */
+	int all_connect;		/* ³(reject ޤ)     */
 	int max_connect;		/* ³߷(reject ޤ) */
 	int overflow;			/* ³¤ۤäˣ */
 };
 
 struct _global_vars {	/* Хѿ */
 	int progrun;				/* 0:ϽλʤФʤʤ0:Ϸ³ */
-	int now_connect;			/* ߤƱ³ϿƤ */
 	struct _total_info total;		/* ߷׾ */
 	struct _server_info server;		/* о */
 	struct _client_info client;		/* ľ³饤Ⱦ */
 	int (*tnetio_run)(int,int,int,char **);	/* ͥΩ³ĤƤνؿ */
 	void *dl_handler;			/* ͭ饤֥ɤ߹ߥϥɥ */
-	int pipe_file_fd;			/* pipe_file Фե뵭һ */
 } G;
 
+#define	PROTO_TCP	0
+#define	PROTO_UDP	1
+
+struct _proto_tbl {
+	const char *name;
+	int type;
+};
+
+const struct _proto_tbl G_proto_tbl[3] =
+{
+	{ "tcp",SOCK_STREAM },
+	{ "udp",SOCK_DGRAM  },
+	{ NULL , 0 }
+};
+
 struct _param_vars {			/* Хѿ(ѥ᡼)¤ */
 	int max_connect;
 	char *bind_addr;
 	int hostnamelookup;
 	char *port;
-	char *proto;
+	int  proto_num;
 	char *env_prefix;
 	int tcp_nodelay;
 	int keepalive;
@@ -105,7 +122,7 @@ const struct _param_vars P_init = {	/* Хѿ(ѥ᡼) */
 	"0.0.0.0",	/* bind_addr */
 	0,		/* hostnamelookup */
 	NULL,		/* port */
-	"tcp",		/* proto */
+	PROTO_TCP,	/* proto_num */
 	"NETSRV_",	/* env_prefix */
 	0,		/* tcp_nodelay */
 	0,		/* keepalive */
@@ -118,33 +135,31 @@ const struct _param_vars P_init = {	/* Хѿ(ѥ᡼) */
 	"netsrv"	/* wrap_name */
 };
 
-extern struct _log_write_param_vars P_log_write;
-
-void signal_handler(int signum)
+static void signal_handler(int signum)
 {
-	log_write(log_msg[LOG_MSG_SIGNAL],signum,G.now_connect,P.max_connect);
+	tnetio_log_write(log_msg[LOG_MSG_SIGNAL],signum,G.total.now_connect,P.max_connect);
 	G.progrun = 0;
 }
 
-void signal_alarm(int signum)
+static void signal_alarm(int signum)
 {
 	/* ä˲⤷ޤ */
 }
 
-void signal_child(int signum)
+static void signal_child(int signum)
 {
 	pid_t child_pid = 0;
 	int status = 0;
 
 	while((child_pid = waitpid(-1,&status,WNOHANG)) > 0)
 	{
-		G.now_connect--;
+		G.total.now_connect--;
 
-		log_write(log_msg[LOG_MSG_CLOSED],child_pid,status,G.now_connect,P.max_connect);
+		tnetio_log_write(log_msg[LOG_MSG_CLOSED],child_pid,status,G.total.now_connect,P.max_connect);
 	}
 }
 
-void signal_wait(void)
+static void signal_wait(void)
 {
 	sigset_t sig_set;
 
@@ -158,7 +173,7 @@ void signal_wait(void)
 	sigsuspend(&sig_set);
 }
 
-int signal_block(int signum)
+static int signal_block(int signum)
 {
 	sigset_t sig_set;
 
@@ -173,7 +188,7 @@ int signal_block(int signum)
 	return sigprocmask(SIG_BLOCK,&sig_set,NULL);
 }
 
-int signal_unblock(int signum)
+static int signal_unblock(int signum)
 {
 	sigset_t sig_set;
 
@@ -188,7 +203,7 @@ int signal_unblock(int signum)
 	return sigprocmask(SIG_UNBLOCK,&sig_set,NULL);
 }
 
-void set_signals(void)
+static void set_signals(void)
 {
 	signal(SIGINT,signal_handler);
 	signal(SIGTERM,signal_handler);
@@ -197,7 +212,7 @@ void set_signals(void)
 }
 
 /* set_signals() ǥåȤʥϡƤδؿǸ᤹ */
-void default_signals(void)
+static void default_signals(void)
 {
 	signal(SIGINT,SIG_DFL);
 	signal(SIGTERM,SIG_DFL);
@@ -205,7 +220,7 @@ void default_signals(void)
 	signal(SIGCHLD,SIG_DFL);
 }
 
-int wait_new_connect(int fd,struct sockaddr *addr, socklen_t *addrlen)
+static int wait_new_connect(int fd,struct sockaddr *addr, socklen_t *addrlen)
 {
 	struct _local_vars {
 		fd_set rfds;
@@ -218,13 +233,13 @@ int wait_new_connect(int fd,struct sockaddr *addr, socklen_t *addrlen)
 
 	while(G.progrun)
 	{
-		if( G.now_connect >= P.max_connect )
+		if( G.total.now_connect >= P.max_connect )
 		{
 			/* ³ޤãƤʤԤޤ */
 			/* ͽۤ륷ʥϡ
 			   (1) SIGTERM (G.progrun = 0)
 			   (2) SIGINT  (G.progrun = 0)
-			   (3) SIGCHLD (G.now_connect--)
+			   (3) SIGCHLD (G.total.now_connect--)
 			   (4) SIGALARM (1÷в)
 			   ΥʥǤ                       */
 			G.total.overflow = 1;	/* Сեȯ */
@@ -243,7 +258,7 @@ int wait_new_connect(int fd,struct sockaddr *addr, socklen_t *addrlen)
 			{
 				if(FD_ISSET(fd, &(l.rfds)))
 				{
-					if((l.newfd = r_eval("accept",accept(fd,addr,addrlen))) >= 0)
+					if((l.newfd = tnetio_accept(fd,addr,addrlen)) >= 0)
 						break;	/* ǽλ */
 					else
 						G.progrun = 0;	/* ̿Ū꤬롣ץλ */
@@ -255,7 +270,7 @@ int wait_new_connect(int fd,struct sockaddr *addr, socklen_t *addrlen)
 	return l.newfd;
 }
 
-int logging_connect(int acs)
+static int logging_connect(int acs)
 {
 	char *fmt = NULL;
 
@@ -264,12 +279,12 @@ int logging_connect(int acs)
 	if(acs)
 		fmt = (char *)log_msg[LOG_MSG_ACCEPT];
 
-	log_write(fmt,inet_ntoa(G.client.addr.sin_addr),G.client.hostname,G.now_connect,P.max_connect);
+	tnetio_log_write(fmt,inet_ntoa(G.client.addr.sin_addr),G.client.hostname,G.total.now_connect,P.max_connect);
 
 	return acs;
 }
 
-int setup_client_info(int fd,struct _client_info *pst_client)
+static int setup_client_info(int fd,struct _client_info *pst_client)
 {
 	struct hostent *client_ent = NULL;
 
@@ -279,7 +294,7 @@ int setup_client_info(int fd,struct _client_info *pst_client)
 
 	pst_client->hostname = "";
 
-	if(r_eval("getpeername",getpeername(fd,(struct sockaddr *)&(pst_client->addr),&(pst_client->len))) >= 0)
+	if(tnetio_getpeername(fd,(struct sockaddr *)&(pst_client->addr),&(pst_client->len)) >= 0)
 	{
 		if(P.hostnamelookup)
 		{
@@ -295,104 +310,22 @@ int setup_client_info(int fd,struct _client_info *pst_client)
 	return 0;
 }
 
-int drop_ip_options(int fd)
+static void tnetio_set_uidgid(int i_uid,int i_gid)
 {
-	struct _local_vars {
-		unsigned char optbuf[1024];
-		unsigned char *optptr;
-		int optsize;
-		char logbuf[1024];
-		int logbuf_remain;
-	} INIT_LOCAL(l);
-
-
-	/* ⤷꤬ IP_OPTIONS ꤷƤʤ顢λݥ˵Ͽ */
-
-	l.optsize = sizeof(l.optbuf);
-	if(getsockopt(fd,IPPROTO_IP,IP_OPTIONS,(char *)l.optbuf,&l.optsize) == 0)
-	{
-		if(l.optsize > 0)
-		{
-			/* ϡIP_OPTIONS ͭˤƤ롣λΥǡ˻Ĥ*/
-
-			l.logbuf[0] = (char)('\0');
-			l.logbuf_remain = sizeof(l.logbuf) - strlen(l.logbuf) - 1;
-
-			l.optptr = l.optbuf;
-			for(;l.optsize > 1;l.optsize--,l.optptr++)
-			{
-				snprintf(l.logbuf + strlen(l.logbuf),l.logbuf_remain,"%02X:",*l.optptr);
-				l.logbuf_remain = sizeof(l.logbuf) - strlen(l.logbuf) - 1;
-				if(l.logbuf_remain <= 20)
-					break;	/* ǰΤ */
-			}
-			snprintf(l.logbuf + strlen(l.logbuf),l.logbuf_remain,"%02X",*l.optptr);	/* Ǹ */
-
-			log_write(log_msg[LOG_MSG_W_IPOPT],l.logbuf);	/* ؽ񤭽Ф */
-		}
-	}
-
-	/* IP_OPTIONS ̵ѤȤIP 롼ƥ󥰤̵ˤʤ(Ϥ) */
-	return setsockopt(fd,IPPROTO_IP,IP_OPTIONS,NULL,0);
-}
-
-void tnetlib_set_tcp_nodelay(int fd)
-{
-	int optval = 1;
-
-
-	if(P.tcp_nodelay)
-	{
-		setsockopt(fd,IPPROTO_TCP,1 /* TCP_NODELAY */,&optval,sizeof(optval));
-	}
-}
-
-void tnetlib_set_keepalive(int fd)
-{
-	int optval = 1;
-
-
-	if(P.keepalive)
-	{
-		setsockopt(fd,SOL_SOCKET,SO_KEEPALIVE,&optval,sizeof(optval));
-	}
-}
-
-void tnetlib_set_linger(int fd,int t_tim)
-{
-	struct linger st_linger;
-
-
-	/* ̤ѥåȤ֤ǡҥץƥλ
-	   (close())ϡꤵ줿Ԥä close() 褦˻Ÿ롣  */
-	st_linger.l_onoff = 1;
-	st_linger.l_linger = t_tim;	/* Linux  man setsockopt ˤȡ 1/100  ñ̤餷 */
-	setsockopt(fd,SOL_SOCKET,SO_LINGER,&(st_linger),sizeof(st_linger));
-}
-
-void tnetlib_set_reuse(int fd)
-{
-	int optval = 1;
-
-	setsockopt(fd,SOL_SOCKET,SO_REUSEADDR,&optval,sizeof(optval));
-}
-
-void tnetlib_set_uidgid(int i_uid,int i_gid)
-{
-	if(i_uid >= 0)
-		setuid(i_uid);
-
 	if(i_gid >= 0)
 		setgid(i_gid);
+
+	if(i_uid >= 0)
+		setuid(i_uid);
 }
 
-int set_my_env(void)
+static int tnetio_set_my_env(void)
 {
 	char env_name[1024];
 	char env_val[1024];
 
 	snprintf(env_name,sizeof(env_name)-1,"%sPROTO",P.env_prefix);
-	snprintf(env_val ,sizeof(env_val)-1 ,"%s",P.proto);
+	snprintf(env_val ,sizeof(env_val)-1 ,"%s",G_proto_tbl[P.proto_num].name);
 	setenv(env_name,env_val,1);
 
 	snprintf(env_name,sizeof(env_name)-1,"%sLOCALIP",P.env_prefix);
@@ -403,25 +336,29 @@ int set_my_env(void)
 	snprintf(env_val ,sizeof(env_val)-1 ,"%d",ntohs(G.server.addr.sin_port));
 	setenv(env_name,env_val,1);
 
-	snprintf(env_name,sizeof(env_name)-1,"%sREMOTEIP",P.env_prefix);
-	snprintf(env_val ,sizeof(env_val)-1 ,"%s",inet_ntoa(G.client.addr.sin_addr));
-	setenv(env_name,env_val,1);
-
-	snprintf(env_name,sizeof(env_name)-1,"%sREMOTEPORT",P.env_prefix);
-	snprintf(env_val ,sizeof(env_val)-1 ,"%d",ntohs(G.client.addr.sin_port));
-	setenv(env_name,env_val,1);
-
-	if(G.client.hostname != NULL)
+	/* ʲδĶѿϡTCP Լλ */
+	if(P.proto_num == PROTO_TCP)
 	{
-		snprintf(env_name,sizeof(env_name)-1,"%sREMOTEHOST",P.env_prefix);
-		snprintf(env_val ,sizeof(env_val)-1 ,"%s",G.client.hostname);
+		snprintf(env_name,sizeof(env_name)-1,"%sREMOTEIP",P.env_prefix);
+		snprintf(env_val ,sizeof(env_val)-1 ,"%s",inet_ntoa(G.client.addr.sin_addr));
 		setenv(env_name,env_val,1);
+
+		snprintf(env_name,sizeof(env_name)-1,"%sREMOTEPORT",P.env_prefix);
+		snprintf(env_val ,sizeof(env_val)-1 ,"%d",ntohs(G.client.addr.sin_port));
+		setenv(env_name,env_val,1);
+
+		if(G.client.hostname != NULL)
+		{
+			snprintf(env_name,sizeof(env_name)-1,"%sREMOTEHOST",P.env_prefix);
+			snprintf(env_val ,sizeof(env_val)-1 ,"%s",G.client.hostname);
+			setenv(env_name,env_val,1);
+		}
 	}
 
 	return 0;
 }
 
-int calc_argv(char **run_child_name)
+static int calc_argv(char **run_child_name)
 {
 	int i;
 
@@ -430,52 +367,56 @@ int calc_argv(char **run_child_name)
 	return i;
 }
 
-void fork_child(int *srvfd,int *newfd,char **run_child_name)
+static void fork_child(int srvfd,int newfd,char **run_child_name)
 {
 	int i_retval = EXIT_REFUSED;
 
 
-	close_if_open(srvfd);	/* ΥåȤĤޤ */
+	close(srvfd);	/* ΥåȤĤޤ */
 
 	default_signals();	/* ʥϰöᤷޤ */
 
 	/* IP_OPTIONS ϡ̵Ȥ롣
 	   ꤬ IP 롼ƥ󥰺ΤԤäƤǽ뤫*/
-	drop_ip_options(*newfd);
+	tnetio_drop_ip_options(newfd);
 
 	/* 饤Ⱦμ(ȴĶѿ) */
-	setup_client_info(*newfd,&(G.client));
+	setup_client_info(newfd,&(G.client));
 
 	/* tnetio_test_client_connect() ƤӽФꤵ줿 IP ³ľ֤Ĵ٤ */
 
 	/* ꤵ줿 IP ɥ쥹³϶ؤƤ(Ȳꤹ) */
 	i_retval = EXIT_REFUSED;
 
-	if(logging_connect(tnetio_test_client_connect(P.reject_file,P.wrap_flag,P.wrap_name,*newfd)))
+	if(logging_connect(tnetio_test_client_connect(P.reject_file,P.wrap_flag,P.wrap_name,newfd)))
 	{
 		/* ꤵ줿 IP ɥ쥹³ϵĤƤޤ */
 		/* åȤγƼ */
-		tnetlib_set_tcp_nodelay(*newfd);
-		tnetlib_set_keepalive(*newfd);
-		tnetlib_set_linger(*newfd,60*100);
+		if(P.tcp_nodelay)
+			tnetio_set_tcp_nodelay(newfd);
+
+		if(P.keepalive)
+			tnetio_set_keepalive(newfd);
+
+		tnetio_set_linger(newfd,60*100);
 
 		/* ³δĶѿ */
-		set_my_env();
+		tnetio_set_my_env();
 
 		/* ؿƤӽФޤ */
-		i_retval = (*G.tnetio_run)(*newfd,*newfd,calc_argv(run_child_name),run_child_name);
+		i_retval = (*G.tnetio_run)(newfd,newfd,calc_argv(run_child_name),run_child_name);
 	}
 
-	close_if_open(newfd);	/* åȤƤʤĤޤ */
+	close(newfd);	/* åȤƤʤĤޤ */
 
 	exit(i_retval);	/* ǽޤ*/
 }
 
-int parse_args(int argc,char *argv[])
+static int parse_args(int argc,char *argv[])
 {
 	struct _local_vars {
 		int c;
-		FILE *fp;
+		int log_fd;
 	} INIT_LOCAL(l);
 
 	do {
@@ -490,33 +431,37 @@ int parse_args(int argc,char *argv[])
 					P.bind_addr = optarg;
 					break;
 				case 'D':	/* ɸϤ˥񤭽Фˡդղäʤ*/
-					P_log_write.stdout_date = 0;
+					tnetio_log_date(0);
 					break;
-				case 'E':	/* ץ¹ԻꤹĶѿƬ optarg ˤ */
+				case 'E':	/* ץ¹ԻꤹĶѿƬ optarg ˤ(TCPΤͭ) */
 					P.env_prefix = optarg;
 					break;
-				case 'l':	/* ³줿 IP ɥ쥹顢ۥ̾θԤ */
+				case 'l':	/* ³줿 IP ɥ쥹顢ۥ̾θԤ(TCPΤͭ) */
 					P.hostnamelookup = 1;
 					break;
-				case 'M':	/* Ʊ³ */
+				case 'M':	/* Ʊ³(TCPΤͭ) */
 					P.max_connect = atoi(optarg);
 					if(P.max_connect <= 0)
 						P.max_connect = 1;
 					break;
-				case 'N':	/* TCP_NODELAY ͭˤ */
+				case 'N':	/* TCP_NODELAY ͭˤ(TCPΤͭ) */
 					P.tcp_nodelay = 1;
 					break;
 				case 'S':	/* ǡ syslog ˽񤭹 */
-					P_log_write.syslog_write = 1;
+					tnetio_log_open();	/* System log open */
 					break;
-				case 'U':	/* UDP Ԥ(̤б) */
-					P.proto = "udp";
+				case 'U':	/* UDP Ԥ */
+					P.proto_num = PROTO_UDP;
 					break;
-				case 'W':	/* tcpwrapper(libwrap) εǽѤ롣ǡץ̾ optarg ˤ */
-					P.wrap_flag = 1;
-					P.wrap_name = optarg;
+				case 'W':	/* tcpwrapper(libwrap) εǽѤ롣ǡץ̾ optarg ˤ(TCPΤͭ) */
+					if(tnetio_libwrap_use())
+					{
+						/* TCP Wrapper 饤֥꤬ѤǤ */
+						P.wrap_flag = 1;
+						P.wrap_name = optarg;
+					}
 					break;
-				case 'X':	/* ͥΩoptarg Ǽե뤬ɤ߽Фǽ¸ߤƤϤǤ */
+				case 'X':	/* ͥΩoptarg Ǽե뤬ɤ߽Фǽ¸ߤƤϤǤ(TCPΤͭ) */
 					P.reject_file = optarg;
 					break;
 				case 'u':	/* ꤷ桼IDڤؤư */
@@ -525,30 +470,30 @@ int parse_args(int argc,char *argv[])
 				case 'g':	/* ꤷ롼IDڤؤư */
 					P.chg_gid = atoi(optarg);
 					break;
-				case 'k':	/* KEEPALIVE ͭˤ */
+				case 'k':	/* KEEPALIVE ͭˤ(TCPΤͭ) */
 					P.keepalive = 1;
 					break;
 				case 'a':	/* ϤɸϤǤϤʤoptarg եɲä */
-					l.fp = fopen(optarg,"ab");
-					if(l.fp == NULL)
+					l.log_fd = open(optarg,(O_CREAT | O_APPEND),(S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH));
+					if(l.log_fd < 0)
 					{
-						perror(optarg);
+						tnetio_log_write(" ABORT : log-file : %s\n",strerror(errno));
 						exit(EXIT_EPERM);
 					}
-					dup2(fileno(l.fp),1);	/* ɸϤϡƤΥե˸ */
-					fclose(l.fp);
+					dup2(l.log_fd,1);	/* ɸϤϡƤΥե˸ */
+					close(l.log_fd);
 					break;
 				case 's':	/* Ϳץ̾ϡץ̾ǤϤʤͭ饤֥ */
 					if(!tnetio_dluse())
 					{
-						/* ͭ饤֥굡ǽϻѤǤޤ */
-						fprintf(stderr,"Not supported : Dynamic-load library function.\n");
-						exit(EXIT_EPERM);
+						exit(EXIT_EPERM); /* ͭ饤֥굡ǽϻѤǤޤ */
 					}
 					P.dl_flag = 1;
 					break;
 				case 'v':	/* Сϸ塢exit(0) */
-					printf("%s : Version %x.%02x (BUILD DATE:%s %s)\n",
+					tnetio_log_pid(0);
+					tnetio_log_date(0);
+					tnetio_log_write("%s : Version %x.%02x (BUILD DATE:%s %s)\n",
 						argv[0],
 						(unsigned)(TNETIO_VERSION >> 8),
 						(unsigned)(TNETIO_VERSION & 0x00ff),
@@ -556,7 +501,7 @@ int parse_args(int argc,char *argv[])
 						__TIME__);
 					exit(0);
 					break;
-				case 'P':	/* ơեˡ߾񤭹褦ˤ */
+				case 'P':	/* ơեˡ߾񤭹褦ˤ(TCPΤͭ) */
 					P.status_file = optarg;
 					break;
 				default:
@@ -570,7 +515,7 @@ int parse_args(int argc,char *argv[])
 	return optind;
 }
 
-void *tnetio_dynamic_load(char *libname)
+static void *tnetio_dynamic_load(char *libname)
 {
 	void *ptr_retval;
 
@@ -589,39 +534,39 @@ void *tnetio_dynamic_load(char *libname)
 
 	if(ptr_retval == NULL)
 	{
-		log_write(log_msg[LOG_MSG_E_DLOAD],libname,tnetio_dlerror());
+		tnetio_log_write(log_msg[LOG_MSG_E_DLOAD],libname,tnetio_dlerror());
 	}
 
 	return ptr_retval;
 }
 
-void tnetio_dynamic_unload(void)
+static void tnetio_dynamic_unload(void)
 {
 	if(G.dl_handler != NULL)
 		tnetio_dlclose(G.dl_handler);
 }
 
-pid_t count_new_connect(pid_t c_pid)
+static pid_t count_new_connect(pid_t c_pid)
 {
 	switch(c_pid)
 	{
 		case 0:		/* ҥץξ */
 			/* ҥץ¦ϡҥץƤʤΤǡ
-			   G.now_connect ͤϡ˽롣                */
-			G.now_connect++;	/* ҥץܣ */
+			   G.total.now_connect ͤϡ˽롣                */
+			G.total.now_connect++;	/* ҥץܣ */
 			break;
 		case -1:	/* ʬȤνѤ˼ԤߤǤ㥯餬­ʤäʡ */
-			log_write(log_msg[LOG_MSG_E_FORK],strerror(errno));
+			tnetio_log_write(log_msg[LOG_MSG_E_FORK],strerror(errno));
 			break;
 		default:	/* ƥץξ */
-			/* 衢ҥץοѲ(G.now_connect ͤѤ)
-			   ΤǡSIGCHLD ʥ뤬ȯȡG.now_connect ζ礬롣 
+			/* 衢ҥץοѲ(G.total.now_connect ͤѤ)
+			   ΤǡSIGCHLD ʥ뤬ȯȡG.total.now_connect ζ礬롣 
 			   򿩤ߤ뤿ˡSIGCHLD ʥȯϡԤäƤ餦 */
 			if(signal_block(SIGCHLD) == 0)
 			{
 				/*  ϡSIGCLHD γߤʤϤ  */
 
-				G.now_connect++;	/* ҥץܣ */
+				G.total.now_connect++;	/* ҥץܣ */
 
 				/* SIGCHLD ʥĤޤ */
 				signal_unblock(SIGCHLD);
@@ -629,9 +574,9 @@ pid_t count_new_connect(pid_t c_pid)
 			}
 
 			/* ³߷פη׻ */
-			G.total.connect++;		/* ³߷סܣ */
-			if(G.now_connect > G.total.max_connect)
-				G.total.max_connect = G.now_connect;	/* Ʊ³߷סܣ */
+			G.total.all_connect++;		/* ³סܣ */
+			if(G.total.now_connect > G.total.max_connect)
+				G.total.max_connect = G.total.now_connect;	/* Ʊ³߷סܣ */
 
 			break;
 	}
@@ -639,13 +584,89 @@ pid_t count_new_connect(pid_t c_pid)
 	return c_pid;
 }
 
+static int job_tcp(int sockfd,char **run_child_name)
+{
+	int newsockfd = -1;
+	int i_retval = EXIT_BINDERR;
+
+
+	/* åȥ塼 */
+	if(tnetio_listen(sockfd,5) >= 0)
+	{
+		/* λ */
+
+		i_retval = EXIT_NORMAL;
+
+		/* ᥤ롼 */
+		while((newsockfd = wait_new_connect(sockfd,NULL,NULL)) >= 0)
+		{
+			/* 줫Ĥʤޤ*/
+
+			/* ץʣ(fork())³򥫥Ȥ */
+			if(count_new_connect(fork()) == 0)
+			{
+				/* ҥץξ硢ҥץν */
+				fork_child(sockfd,newsockfd,run_child_name);
+			}
+
+			close(newsockfd);
+			newsockfd = -1;
+		}
+	}
+
+	return i_retval;
+}
+
+static int job_udp(int sockfd,char **run_child_name)
+{
+	int i_retval = 0;
+
+	/* UDP Լξ硢TCP ¦ǹԤۤȤɤΥӥѤǤʤ*/
+
+	default_signals();	/* ʥϰöᤷޤ */
+
+	/* ³δĶѿ */
+	tnetio_set_my_env();
+
+	while(i_retval >= 0)
+	{
+		/* ؿ򤤤ʤƤӽФޤ֤ʤСλޤ */
+		i_retval = (*G.tnetio_run)(sockfd,sockfd,calc_argv(run_child_name),run_child_name);
+	}
+
+	return i_retval;
+}
+
+static int job_main(int sockfd,char **run_child_name)
+{
+	int i_retval = 0;
+
+
+	if(P.proto_num == PROTO_TCP)
+		i_retval = job_tcp(sockfd,run_child_name);
+	else
+		i_retval = job_udp(sockfd,run_child_name);
+
+	return i_retval;
+}
+
+static int init_param(char *myname)
+{
+	/* Хѿ(ѥ᡼)ν */
+	memcpy(&P,&P_init,sizeof(P));
+
+	/* طν */
+	tnetio_log_init(myname);
+
+	return 0;
+}
+
 int main(int argc,char *argv[])
 {
 	struct _local_vars {
 		int start_arg;
 		char **run_child_name;
 		int sockfd;
-		int newsockfd;
 		int i_retval;
 	} INIT_LOCAL(l);
 
@@ -653,14 +674,8 @@ int main(int argc,char *argv[])
 	memset(&G,0x00,sizeof(G));
 	G.progrun  = 1;
 
-	/* Хѿ(ѥ᡼)ν */
-	memcpy(&P,&P_init,sizeof(P));
-
-	/* طν */
-	P_log_write.syslog_write = 0;	/* ɸϤ˽Ф */
-	P_log_write.syslog_ident = argv[0];
-	if(rindex(argv[0],'/') != NULL)
-		P_log_write.syslog_ident = rindex(argv[0],'/') + 1;
+	/* ѥ᡼ѿν */
+	init_param(argv[0]);
 
 	/* ץβ */
 	l.start_arg = parse_args(argc,argv);
@@ -673,20 +688,19 @@ int main(int argc,char *argv[])
 
 	/* λǡbind  IPݡֹ桢ץȥ뤬Ƥ
 	   Τǡbind 뤿ξ󤬹ۤǤ뤫Ĵ٤롣                */
-	switch(tnetio_sockaddr_in_byname(P.hostnamelookup,P.bind_addr,P.port,P.proto,&(G.server.addr)))
+	switch(tnetio_sockaddr_in_byname(P.hostnamelookup,P.bind_addr,P.port,(char *)(G_proto_tbl[P.proto_num].name),&(G.server.addr)))
 	{
 		case 0:		/*  */
+			G.server.type = G_proto_tbl[P.proto_num].type;
 			break;
 		case -1:	/* bind ۥ̾ */
-			fprintf(stderr,"%s : Invalid bind-addr %s\n",
-				P_log_write.syslog_ident,
+			tnetio_log_write(" ABORT : Invalid bind-addr %s\n",
 				(P.bind_addr != NULL) ? P.bind_addr : "(none)"
 				);
 			return EXIT_EPERM;
 			break;
 		case -2:	/* bind ݡֹ椬 */
-			fprintf(stderr,"%s : Invalid bind-port %s\n",
-				P_log_write.syslog_ident,
+			tnetio_log_write(" ABORT : Invalid bind-port %s\n",
 				(P.port != NULL) ? P.port : "(none)"
 				);
 			return EXIT_EPERM;
@@ -706,7 +720,7 @@ int main(int argc,char *argv[])
 
 	if( (l.run_child_name == NULL ) || (strlen(*(l.run_child_name + 0)) <= 0) )
 	{
-		fprintf(stderr,"%s : no child-program\n",P_log_write.syslog_ident);
+		tnetio_log_write(" ABORT : no child-program\n");
 		return EXIT_EPERM;
 	}
 
@@ -715,58 +729,42 @@ int main(int argc,char *argv[])
 	l.i_retval = EXIT_BINDERR;
 
 	/* åȤ򳫤 */
-	if((l.sockfd = r_eval("socket",socket(G.server.addr.sin_family,SOCK_STREAM,0))) >= 0)
+	if((l.sockfd = tnetio_socket(G.server.addr.sin_family,G.server.type,0)) >= 0)
 	{
-		tnetlib_set_reuse(l.sockfd);	/* 륢ɥ쥹κѤǽˤƤޤ */
+		tnetio_set_reuse(l.sockfd);	/* 륢ɥ쥹κѤǽˤƤޤ */
 
 		/* åȤ bind  */
-		if(r_eval("bind",bind(l.sockfd,(struct sockaddr *)&(G.server.addr),sizeof(G.server.addr))) >= 0)
+		if(tnetio_bind(l.sockfd,(struct sockaddr *)&(G.server.addr),sizeof(G.server.addr)) >= 0)
 		{
 			/* bind 褿顢桼 ID ȡ롼 ID ڤؤ롣ʹߤưϡʲθ¤ǹԤ */
-			tnetlib_set_uidgid(P.chg_uid,P.chg_gid);
+			tnetio_set_uidgid(P.chg_uid,P.chg_gid);
 
 			/* ꤵ줿ץ̾¤϶ͭ饤֥Ǥ硢ɤԤ */
+			l.i_retval = EXIT_SOLOAD;
 			if( (G.tnetio_run = tnetio_dynamic_load(*(l.run_child_name + 0))) != NULL )
 			{
-				/* åȥ塼 */
-				if(r_eval("listen",listen(l.sockfd,5)) >= 0)
-				{
-					/* λ */
+				/* ä */
+				tnetio_log_write(log_msg[LOG_MSG_BIND],
+					inet_ntoa(G.server.addr.sin_addr),
+					ntohs(G.server.addr.sin_port),
+					G_proto_tbl[P.proto_num].name,
+					*(l.run_child_name + 0));
 
-					log_write(log_msg[LOG_MSG_BIND],
-						inet_ntoa(G.server.addr.sin_addr),
-						ntohs(G.server.addr.sin_port),
-						P.proto,
-						*(l.run_child_name + 0));
+				tnetio_log_write(log_msg[LOG_MSG_WAIT],G.total.now_connect,P.max_connect);
 
-					log_write(log_msg[LOG_MSG_WAIT],G.now_connect,P.max_connect);
+				l.i_retval = job_main(l.sockfd,l.run_child_name);
 
-					l.i_retval = EXIT_NORMAL;
-
-					/* ᥤ롼 */
-					while((l.newsockfd = wait_new_connect(l.sockfd,NULL,NULL)) >= 0)
-					{
-						/* 줫Ĥʤޤ*/
-
-						/* ץʣ(fork())³򥫥Ȥ */
-						if(count_new_connect(fork()) == 0)
-						{
-							/* ҥץξ硢ҥץν */
-							fork_child(&(l.sockfd),&(l.newsockfd),l.run_child_name);
-						}
-
-						close_if_open(&(l.newsockfd));
-					}
-				}
 			}
 
 			tnetio_dynamic_unload();
 		}
 
-		close_if_open(&(l.sockfd));
-
+		close(l.sockfd);
+		l.sockfd = -1;
 	}
 
+	tnetio_log_close();
+
 	return l.i_retval;
 }
 
